% ########################################################################
% Eine erste Funktion in Octave.
% ########################################################################
% - Funktionen sind wichtig fuer einen strukturierten Programmaufbau. Man
% kann sie sich als Kurzschreibweise fuer wiederkehrenden Programmcode
% vorstellen.
% Eine Funktion kann Ein- und Ausgabevariablen haben.
% Eine Funktion hat eine erste und letzte Zeile, die mit "function" bzw mit
% "endfunction" beginnen. Dazwischen befindet sich die Funktionsdeklaration. 
% Innerhalb der Deklaration koennen Eingabevariablen wie Konstanten verwendet
% werden, insbesondere kann man sie auch ueberschreiben, wenn man ihren
% Wert nicht mehr braucht.
% Darueber hinaus sind innerhalb der Funktionsdeklaration keine Variablen
% von ausserhalb der Deklaration verfuegbar! Alle Werte die man braucht muessen
% also ueber Eingabevariablen uebergeben werden.
% Ist der Code der Funktion abgearbeitet werden die berechneten Werte der
% Ausgabevariablen an das Programm von dem aus man die Funktion aufgerufen hatte
% zurueckgegeben. Dies stellt die einzige Moeglichkeit dar, wie Informationen
% aus einer Funktion herauskommen koennen. Es ist also moeglich innerhalb einer
% Funktionsdeklarationen Variablen zu deklarieren, die den Namen einer
% Variablen ausserhalb der Funktion tragen. Diese haben dann aber nichts
% miteinander zu tun!
% Eine Funktion wird in der Regel in einer Datei mit Endung .m definiert,
% deren Name mit dem Namen der Funktion uebereinstimmt.
% ########################################################################

% ########################################################################
% Syntax:
%
% function [a1, a2, ...] = NAME(e1, e2, ...)
% DEKLARATION
% endfunction
%
% a1,a2 usw bezeichnen Ausgabevariablen. Eine Funktion muss keine haben!
% In so einem Fall ist die Liste leer und kann weggelassen werden.
% (man laesst dann natuerlich auch das "=" weg)
% e1,e2 usw sind Eingabevariablen.. Gibt es keine, kann man entweder
% "NAME()" oder "NAME" schreiben.
% DEKLARATION - hier kommt euer Beitrag

% Funktionen werden mit 
% name(e1,e2,..);
% oder
% [a1 a2 ...] = name(e1,e2,...);
% aufgerufen
% ########################################################################


% Falls folgende Zeile aktiviert wird, so wird aus der Funktionsdatei
% ein Skript. Zwischen Skripten und Funktionsdateien muessen wir strikt
% unterscheiden. Eine Funktionsdatei beginnt stets mit dem Schluesselwort
% function.
%u=1



% Eine erste Funktion ohne Ein- und Ausgabevariablen.
% In der Funktionsdatei myfunction.m muss als erstes nun auch die Funktion
% namens myfunction definiert werden
function myfunction
  % Mit printf erzeugt man formatierten Output auf der Octave-Konsole.
  % Es gibt auch den Befehl sprintf, der im wesentlichen wie printf funktioniert,
  % aber einen String (Zeichenkette) zurueckliefert.
  printf("##########################################################\n")
  printf("Hallo Welt! Ich bin eine triviale Funktion in Octave\n")
  printf("##########################################################\n\n")

  a=2;

  % auch nicht schlecht:
  %a=[2 -1; 5 0];
  printf("Eine lokale Variable wird auf %i gesetzt und an eine Unterfunktion weitergereicht ...\n", a)
  y=slave1(a);
  printf(" ... Der Wert der uebergebenen Variable nach dem Aufruf der Unterfuntion ist %i\n\n", a)
  printf("Die Unterfunktion gibt den Wert y=%3.10f zurueck.\n", y)

  % WICHTIG: Variablen, die im Octave-Workspace vorhanden sind, sind hier
  % nicht sichtbar.
  % Hat man also auf der Octave-Konsole z.B. eine Matrix A=ones(3,3)
  % definiert, so kann man hier nicht auf A zurueckgreifen. Der Aufruf
  %A
  % wuerde an dieser Stelle einen Fehler produzieren.
  % A=[4 4]
  % ist erlaubt, ueberschreibt aber nicht das im Workspace vorhandene A.
endfunction


% Eine erste Funktion mit Ausgabevariable 'y' und
% Eingabevariable 'x'
% Diese Funktion ist auf der Octave-Konsole nicht sichtbar!!!
function y=slave1(x)
  y=x.^2;
  % Die folgende Anweisung hat nuer einen lokalen Effekt.
  % Der Wert der in 'myfunction' uebergebenen Variable 'a' wird
  % hierdurch nicht veraendert.
  x=0;
endfunction


% ########################################################################
% Abwandlung: 
% - u=1 einkommentieren, auf Konsole clear all, myfunction aufrufen.
%     Erhalten:
%     u =  1
%      warning: function `myfunction' defined within script file
%        `/home/werner/PROMOTION/Vorlesungen/NumerikI_09s/Octave/Tutorium2/myfunction.m'
% - Dann nochmal myfunction aufrufen. Problem: Fehler, da  slave1 unbekannt!
%   Und das obwohl slave1 jetzt im Workspace liegt und auf der Konsole benutzt werden kann.
% - Dann slave1 vor myfunction kopieren. myfunction auf Konsole aufrufen.
%   Hilft nichts!
% - myfunction in myfunction2 umbenennen. clear all auf Konsole. myfunction auf Konsole
%   aufrufen. Danach sofort myfunction2 auf Konsole aufrufen. Alles funktioniert.
%   Sogar slave1 ist dann auf der Konsole bekannt. Funktionen, die in Skripten
%   statt in Funktionsdateien definiert werden, werden also auch dem Octave-Workspace
%   hinzugefueft.
% - Dann slave1 unter myfunction2 kopieren. clear all auf der Konsole.
%   myfunction2 aufrufen. Unbekannt. (Klar.) myfunction aufrufen.
%   myfunction2 aufrufen. Alles funktioniert.
% Mit who -functions kann man sich uebrigens alle im Workspace vorhandenen
% Funktionen anzeigen lassen.
% ########################################################################
